@@ -193,7 +193,8 @@ int setenv(const char* name, const char* value, int overwrite)
 	if(!t) return -1;
 	sprintf(t, "%s=%s", name, value);
 	r = putenv(t);
-	free(t);
+	if(r != 0)
+		free(t);
 	return r;
 }
 
@@ -254,6 +255,96 @@ int daemon(int nochdir, int noclose)
 
 #endif
 
+#ifndef HAVE_GETLINE
+
+ssize_t getline(char** lineptr, size_t* n, FILE* stream)
+{
+    return getdelim(lineptr, n, '\n', stream);
+}
+
+#endif
+
+#ifndef HAVE_GETDELIM
+
+ssize_t getdelim(char** lineptr, size_t* n, int delim, FILE* stream)
+{
+    size_t written = 0;
+    int allocated, ch;
+    ssize_t ret = -1;
+    char* p;
+    
+    if(!n || !lineptr || !stream)
+    {
+        errno = EINVAL;
+        return -1;
+    }
+    
+    /* Track whether we allocated this or not */
+    allocated = !(*lineptr);
+    
+#ifdef HAVE_FLOCKFILE
+    /* Affects performance on Solaris. */
+    flockfile(stream);
+#endif
+    
+    for(;;) 
+    {
+        if(!*lineptr)
+            *n = 0;
+        
+        /* Reallocate if we need more space */
+        if(written == *n - 1) 
+        {
+            *n = *n ? *n * 2 : 256;
+            if(!(p = (char*)realloc(*lineptr, *n)))
+            {
+                if(allocated && *lineptr)
+                    free(*lineptr);
+                errno = ENOMEM;
+                ret = -1;
+                goto finally;
+            }
+            *lineptr = p;
+        }
+        
+        while(written < *n - 1)
+        {
+#ifdef HAVE_FLOCKFILE
+            ch = getc_unlocked(stream);
+#else
+            ch = fgetc(stream);
+#endif
+
+            if(ferror(stream)) 
+            {
+                if(allocated && *lineptr)
+                    free(*lineptr);
+                ret = -1;
+                goto finally;
+            }
+
+            if(ch != EOF)
+                (*lineptr)[written++] = ch;
+            
+            /* Done, null terminate, return */
+            if(ch == EOF || ch == delim)
+            {
+                (*lineptr)[written] = 0;
+                ret = written ? written : -1;
+                goto finally;
+            }
+        }
+    }    
+finally:
+    
+#ifdef HAVE_FLOCKFILE
+    funlockfile(stream);
+#endif
+    return ret;
+}
+
+#endif
+
 #ifndef HAVE_ERR_H
 
 #include <stdlib.h>
