@@ -44,6 +44,7 @@
 #endif
 
 #include <sys/types.h>
+#include <stdlib.h>
 
 #ifndef HAVE_STDARG_H
 #error ERROR: Must have a working stdarg.h header
@@ -51,6 +52,10 @@
 #include <stdarg.h>
 #endif
 
+#ifdef WITH_DMALLOC
+#include "dmalloc.h"
+#endif
+
 #ifndef HAVE_REALLOCF
 void* reallocf(void* p, size_t sz);
 #endif
